package com.coherentlogic.fred.client.core.domain;

import static com.coherentlogic.fred.client.core.util.Constants.ID;
import static com.coherentlogic.fred.client.core.util.Constants.IDENTITY_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.LINK;
import static com.coherentlogic.fred.client.core.util.Constants.NAME;
import static com.coherentlogic.fred.client.core.util.Constants.PRESS_RELEASE;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_END;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_START;
import static com.coherentlogic.fred.client.core.util.Constants.RELEASE;
import static com.coherentlogic.fred.client.core.util.Constants.RELEASE_TABLE;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;

/**
 * A class which represents a release of an economic data series.
 *
 * @see <a href="http://api.stlouisfed.org/docs/fred/releases.html">releases</a>
 * @see <a href="http://api.stlouisfed.org/docs/fred/release.html">release</a>
 * @see <a href="http://api.stlouisfed.org/docs/fred/series_release.html">
 * series_release</a>
 * @see <a href="http://api.stlouisfed.org/docs/fred/source_releases.html">
 * source_releases</a>
 *
 * @author <a href="support@coherentlogic.com">Support</a>
 */
@Entity//@Entity(name=RELEASE_TABLE) NOT SURE WE NEED THE TABLE NAME AT THIS
// TIME, HOWEVER IT WILL STAY HERE AS A HINT IF THE TEST FAILS WHEN WORKING
// WITH MYSQL.
@Table(name=RELEASE_TABLE)
@XStreamAlias(RELEASE)
public class Release extends DefaultObject
    implements IdentitySpecification<Long>, RealtimeBoundSpecification {

    private static final long serialVersionUID = 3429980868680074029L;

    @XStreamAlias(ID)
    @XStreamAsAttribute
    private Long id;

    @XStreamAlias(REALTIME_START)
    @XStreamAsAttribute
    private Date realtimeStart = null;

    @XStreamAlias(REALTIME_END)
    @XStreamAsAttribute
    private Date realtimeEnd = null;

    @XStreamAlias(NAME)
    @XStreamAsAttribute
    private String name = null;

    @XStreamAlias(PRESS_RELEASE)
    @XStreamAsAttribute
    private boolean pressRelease = false;

    @XStreamAlias(LINK)
    @XStreamAsAttribute
    private String link = null;

    @Column(name=IDENTITY_VALUE)
    @Override
    public Long getId() {
        return id;
    }

    @Override
    public void setId(Long id) {
        this.id = id;
    }

    @Override
    public Date getRealtimeStart() {
        return clone (realtimeStart);
    }

    @Override
    public void setRealtimeStart(Date realtimeStart) {
        this.realtimeStart = clone (realtimeStart);
    }

    @Override
    public Date getRealtimeEnd() {
        return clone (realtimeEnd);
    }

    @Override
    public void setRealtimeEnd(Date realtimeEnd) {
        this.realtimeEnd = clone (realtimeEnd);
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public boolean isPressRelease() {
        return pressRelease;
    }

    public void setPressRelease(boolean pressRelease) {
        this.pressRelease = pressRelease;
    }

    public String getLink() {
        return link;
    }

    public void setLink(String link) {
        this.link = link;
    }
}
