package com.coherentlogic.fred.client.core.domain;

import static com.coherentlogic.fred.client.core.util.Constants.COUNT;
import static com.coherentlogic.fred.client.core.util.Constants.COUNT_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.LIMIT;
import static com.coherentlogic.fred.client.core.util.Constants.LIMIT_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.OFFSET;
import static com.coherentlogic.fred.client.core.util.Constants.OFFSET_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.ORDER_BY;
import static com.coherentlogic.fred.client.core.util.Constants.ORDER_BY_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_END;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_START;
import static com.coherentlogic.fred.client.core.util.Constants.RELEASES;
import static com.coherentlogic.fred.client.core.util.Constants.SORT_ORDER;

import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;

/**
 * A class which represents the releases of an economic data series.
 *
 * @see <a href="http://api.stlouisfed.org/docs/fred/releases.html">releases</a>
 * @see <a href="http://api.stlouisfed.org/docs/fred/release.html">release</a>
 * @see <a href="http://api.stlouisfed.org/docs/fred/series_release.html">
 * series_release</a>
 * @see <a href="http://api.stlouisfed.org/docs/fred/source_releases.html">
 * source_releases</a>
 * @see <a href="http://research.stlouisfed.org/fred2/releases/">releases</a>
 *
 * @author <a href="support@coherentlogic.com">Support</a>
 */
@Entity
@Table(name=RELEASES)
@XStreamAlias(RELEASES)
public class Releases extends DefaultObject
    implements RealtimeBoundSpecification, OrderBySpecification, SortOrderSpecification, PaginationSpecification {

    private static final long serialVersionUID = -8795478585031544710L;

    @XStreamImplicit
    private List<Release> releaseList = null;

    @XStreamAlias(REALTIME_START)
    @XStreamAsAttribute
    private Date realtimeStart = null;

    @XStreamAlias(REALTIME_END)
    @XStreamAsAttribute
    private Date realtimeEnd = null;

    @XStreamAlias(COUNT)
    @XStreamAsAttribute
    private int count = 0;

    @XStreamAlias(LIMIT)
    @XStreamAsAttribute
    private long limit = 0;

    @XStreamAlias(OFFSET)
    @XStreamAsAttribute
    private int offset = 0;

    @XStreamAlias(SORT_ORDER)
    @XStreamAsAttribute
    private SortOrder sortOrder = null;

    @XStreamAlias(ORDER_BY)
    @XStreamAsAttribute
    private OrderBy orderBy = null;

    @OneToMany(cascade=CascadeType.ALL)
    public List<Release> getReleaseList () {
        return releaseList;
    }

    public void setReleaseList (List<Release> releaseList) {
        this.releaseList = releaseList;
    }

    @Override
    public Date getRealtimeStart() {
        return clone (realtimeStart);
    }

    @Override
    public void setRealtimeStart(Date realtimeStart) {
        this.realtimeStart = clone (realtimeStart);
    }

    @Override
    public Date getRealtimeEnd() {
        return clone (realtimeEnd);
    }

    @Override
    public void setRealtimeEnd(Date realtimeEnd) {
        this.realtimeEnd = clone (realtimeEnd);
    }

    @Override
    public void setLimit(long limit) {
        this.limit = limit;
    }

    @Override
    public void setOffset(int offset) {
        this.offset = offset;
    }

    /**
     * In the H2 database the limit is a reserved keyword so we are renaming
     * this, otherwise we'll see an exception along the lines of "Table
     * 'DEFAULTOBJECT' not found; SQL statement".
     */
    @Column(name=LIMIT_VALUE)
    @Override
    public long getLimit() {
        return limit;
    }

    @Column(name=OFFSET_VALUE)
    @Override
    public int getOffset() {
        return offset;
    }

    @Override
    public void setSortOrder(SortOrder sortOrder) {
        this.sortOrder = sortOrder;
    }

    @Override
    public SortOrder getSortOrder() {
        return sortOrder;
    }

    @Column(name=ORDER_BY_VALUE)
    @Override
    public OrderBy getOrderBy() {
        return orderBy;
    }

    @Override
    public void setOrderBy(OrderBy orderBy) {
        this.orderBy = orderBy;
    }

    @Column(name=COUNT_VALUE)
    public int getCount() {
        return count;
    }

    public void setCount(int count) {
        this.count = count;
    }
}
