package com.coherentlogic.fred.client.core.domain;

import static com.coherentlogic.fred.client.core.util.Constants.FREQUENCY;
import static com.coherentlogic.fred.client.core.util.Constants.FREQUENCY_SHORT;
import static com.coherentlogic.fred.client.core.util.Constants.
    FREQUENCY_SHORT_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.FREQUENCY_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.LAST_UPDATED;
import static com.coherentlogic.fred.client.core.util.Constants.NOTES;
import static com.coherentlogic.fred.client.core.util.Constants.OBSERVATION_END;
import static com.coherentlogic.fred.client.core.util.Constants.
    OBSERVATION_START;
import static com.coherentlogic.fred.client.core.util.Constants.POPULARITY;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_END;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_START;
import static com.coherentlogic.fred.client.core.util.Constants.
    SEASONAL_ADJUSTMENT;
import static com.coherentlogic.fred.client.core.util.Constants.
    SEASONAL_ADJUSTMENT_SHORT;
import static com.coherentlogic.fred.client.core.util.Constants.SERIES;
import static com.coherentlogic.fred.client.core.util.Constants.TITLE;
import static com.coherentlogic.fred.client.core.util.Constants.UNITS;
import static com.coherentlogic.fred.client.core.util.Constants.UNITS_SHORT;
import static com.coherentlogic.fred.client.core.util.Constants.UNITS_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.
    UNITS_SHORT_VALUE;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;

import com.coherentlogic.fred.client.core.converters.PopularityConverter;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamConverter;

/**
 * A class which represents an economic data series.
 *
 * @see <a href="http://api.stlouisfed.org/docs/fred/series.html">series</a>
 *
 * @author <a href="support@coherentlogic.com">Support</a>
 */
@Entity
@Table(name="series")
@XStreamAlias(SERIES)
public class Series extends IdentityObject implements RealtimeBoundSpecification {

    private static final long serialVersionUID = 3637864692827055592L;

    @XStreamAlias(REALTIME_START)
    @XStreamAsAttribute
    private Date realtimeStart = null;

    @XStreamAlias(REALTIME_END)
    @XStreamAsAttribute
    private Date realtimeEnd = null;

    @XStreamAlias(TITLE)
    @XStreamAsAttribute
    private String title = null;

    @XStreamAlias(OBSERVATION_START)
    @XStreamAsAttribute
    private Date observationStart = null;

    @XStreamAlias(OBSERVATION_END)
    @XStreamAsAttribute
    private Date observationEnd = null;

    @XStreamAlias(FREQUENCY)
    @XStreamAsAttribute
    private String frequency = null;

    @XStreamAlias(FREQUENCY_SHORT)
    @XStreamAsAttribute
    private String frequencyShort = null;

    @XStreamAlias(UNITS)
    @XStreamAsAttribute
    private String units = null;

    @XStreamAlias(UNITS_SHORT)
    @XStreamAsAttribute
    private String unitsShort = null;

    @XStreamAlias(SEASONAL_ADJUSTMENT)
    @XStreamAsAttribute
    private String seasonalAdjustment = null;

    @XStreamAlias(SEASONAL_ADJUSTMENT_SHORT)
    @XStreamAsAttribute
    private String seasonalAdjustmentShort = null;

    @XStreamAlias(LAST_UPDATED)
    @XStreamAsAttribute
    private Date lastUpdated = null;

    @XStreamAlias(POPULARITY)
    @XStreamAsAttribute
    @XStreamConverter(PopularityConverter.class)
    private Integer popularity = null;

    @XStreamAlias(NOTES)
    @XStreamAsAttribute
    private String notes = null;

    public Date getRealtimeStart () {
        return clone (realtimeStart);
    }

    public void setRealtimeStart (Date realtimeStart) {
        this.realtimeStart = clone (realtimeStart);
    }

    public Date getRealtimeEnd () {
        return clone (realtimeEnd);
    }

    public void setRealtimeEnd (Date realtimeEnd) {
        this.realtimeEnd = clone (realtimeEnd);
    }

    public String getTitle () {
        return title;
    }

    public void setTitle (String title) {
        this.title = title;
    }

    public Date getObservationStart () {
        return clone (observationStart);
    }

    public void setObservationStart (Date observationStart) {
        this.observationStart = clone (observationStart);
    }

    public Date getObservationEnd () {
        return clone (observationEnd);
    }

    public void setObservationEnd (Date observationEnd) {
        this.observationEnd = clone (observationEnd);
    }

    /**
     * Getter method for the frequency.
     *
     * @return The frequency as a string.
     *
     * @todo Should this be returning an enum?
     */
    @Column(name=FREQUENCY_VALUE)
    public String getFrequency () {
        return frequency;
    }

    /**
     * Setter method for the frequency.
     *
     * @param frequency The frequency as a string.
     *
     * @todo Should this be using an enum?
     */
    public void setFrequency (String frequency) {
        this.frequency = frequency;
    }

    /**
     * Getter method for the short frequency.
     *
     * @return The short frequency as a string.
     *
     * @todo Should this be returning an enum?
     */
    @Column(name=FREQUENCY_SHORT_VALUE)
    public String getFrequencyShort () {
        return frequencyShort;
    }

    public void setFrequencyShort (String frequencyShort) {
        this.frequencyShort = frequencyShort;
    }

    /**
     * Getter method for the units.
     *
     * @return The units as a string.
     *
     * @todo Should this be returning an enum?
     */
    @Column(name=UNITS_VALUE)
    public String getUnits () {
        return units;
    }

    public void setUnits (String units) {
        this.units = units;
    }

    /**
     * Getter method for the short units.
     *
     * @return The short units as a string.
     *
     * @todo Should this be returning an enum?
     */
    @Column(name=UNITS_SHORT_VALUE)
    public String getUnitsShort () {
        return unitsShort;
    }

    public void setUnitsShort (String unitsShort) {
        this.unitsShort = unitsShort;
    }

    public String getSeasonalAdjustment () {
        return seasonalAdjustment;
    }

    public void setSeasonalAdjustment (String seasonalAdjustment) {
        this.seasonalAdjustment = seasonalAdjustment;
    }

    public String getSeasonalAdjustmentShort () {
        return seasonalAdjustmentShort;
    }

    public void setSeasonalAdjustmentShort (String seasonalAdjustmentShort) {
        this.seasonalAdjustmentShort = seasonalAdjustmentShort;
    }

    public Date getLastUpdated () {
        return clone (lastUpdated);
    }

    public void setLastUpdated (Date lastUpdated) {
        this.lastUpdated = clone (lastUpdated);
    }

    public Integer getPopularity () {
        return popularity;
    }

    public void setPopularity (Integer popularity) {
        this.popularity = popularity;
    }

    public String getNotes () {
        return notes;
    }

    public void setNotes (String notes) {
        this.notes = notes;
    }
}
